CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C       								       C
C       			   GALA v 1.1				       C
C       An automatic tool for the abundance analysis of stellar spectra        C
C       								       C
C     Copyright (C) 2013  Alessio Mucciarelli				       C
C       								       C
C     This program is free software: you can redistribute it and/or modify     C
C     it under the terms of the GNU General Public License as published by     C
C     the Free Software Foundation, either version 3 of the License, or        C
C     (at your option) any later version.				       C
C       								       C
C     This program is distributed in the hope that it will be useful,	       C
C     but WITHOUT ANY WARRANTY; without even the implied warranty of	       C
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	       C
C     GNU General Public License for more details.			       C
C       								       C
C     You should have received a copy of the GNU General Public License        C
C     along with this program.  If not, see <http://www.gnu.org/licenses/>.    C
C       								       C
C     For any questions please contact us: alessio.mucciarelli2@unibo.it       C
C       								       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


      program cheshire 

      implicit none
      include 'Declare.f'      

C----------------------------------------------------------------------   initial setting    

      call system('rm abundance.final')
      
      call system('clear')
      
      write(*,*)'                     ########################'
      write(*,*)'                             GALA 1.1'
      write(*,*)'                            (Apr 2013)'
      write(*,*)
      
      
      
C----- check the existence of the input file 'autofl.param'      

      status = access('autofl.param',' ')
      if(status.ne.0) then 
         write(*,*)'Input file autofl.param does not exist!'
	 stop
      endif	 

C----- check the existence of the input file 'list_star'      
      
      status = access('list_star',' ')
      if(status.ne.0) then 
         write(*,*)'Input file list_star does not exist!'
	 stop
      endif	 
 
 
C----- open the input files

      open(unit=1, file='autofl.param', status='unknown')     
      open(unit=2, file='list_star', status='unknown')
    
	       
C----- read the keywords 

      !--------------------------------------------  parameters
      
      
100     read(1,*,end=101)st,st2
        ik=0
	if(st(1:1).ne.'#')then
	  
	   if(st2(1:1).eq.'-')then
	      ik=ik+1
	   else
	     do i=0,9
	       write(jcar,'(I1)')i
	       if(st2(1:1).eq.jcar)ik=ik+1
	     enddo
	   endif
	   !---'model'
	   if(ik.eq.0)model=st2
	   !---numerical keywords
	   if(ik.ne.0)then
             backspace(1)
	     read(1,*)st,val
    	       if(st.eq.'guess')guess=int(val)      
    	       if(st.eq.'uncer')uncer=int(val)
    	       if(st.eq.'refine')refine=int(val) 
    	       if(st.eq.'qiron')qiron=val 
    	       if(st.eq.'paropt')paropt=int(val) 
    	       if(st.eq.'epot')epot=int(val) 
    	       if(st.eq.'grc')grc=int(val) 
    	       if(st.eq.'vtc')vtc=int(val) 
    	       if(st.eq.'metal')metal=int(val)
    	       if(st.eq.'stepgr')stepgr=abs(val) 
    	       if(st.eq.'stepteff')stepteff=abs(val) 
    	       if(st.eq.'stepvt')stepvt=abs(val)  
    	       if(st.eq.'ewmin')ewmin=val
    	       if(st.eq.'ewmax')ewmax=val  
    	       if(st.eq.'error')error=val 
    	       if(st.eq.'iter')iter=int(val) 
    	       if(st.eq.'rj')rj=int(val)
    	       if(st.eq.'smax')smax=val  
    	       if(st.eq.'minnfe')minnfe=int(val)
    	       if(st.eq.'eplever')eplever=val
    	       if(st.eq.'maxsig')maxsig=val
    	       if(st.eq.'minl')minl=int(val)
    	       if(st.eq.'errfit')errfit=int(val)   
    	       if(st.eq.'interz')interz=int(val)   
    	       if(st.eq.'kuriter')kuriter=int(val)   
    	       if(st.eq.'plot')plot=int(val)       
    	       if(st.eq.'verbose')verbose=int(val)      	 		     
	   endif
	endif
	goto 100
101     continue
    
      
      
C---- the original analysis options are stored in the matrix 'step_true'
      
      step_true(1,1)=grc
      step_true(2,1)=epot
      step_true(3,1)=vtc
      
      close(1)
      
      
      if(stepgr.eq.0)grc=0
      if(stepteff.eq.0)epot=0
      if(stepvt.eq.0)vtc=0
            
C ---------------------------------------------------------------------   

      if(verbose.gt.3)verbose=3
      if(verbose.lt.0)verbose=0
      
   
C ---------------------------------------------------------------------   
      
      if((qiron-nint(qiron)).eq.0.0)then
        qiron2=qiron+0.01
	xioni='I '
      endif	
      if((qiron-nint(qiron)).ne.0.0)then
        qiron2=qiron-0.01
        xioni='II'
      endif
      
      xelem=elemx(nint(qiron))
       
      if(verbose.ge.2)then
      
      write(*,*)'Used model atmospheres:      ',model
      if(paropt.eq.0)write(*,*)'Used optimization parameter: slope'
      if(paropt.eq.1)write(*,*)'Used optimization parameter: Spearman'
      if((qiron-nint(qiron)).eq.0.0)
     &   write(*,'(A23,A2,A8)')' Optimization based on ',
     &   xelem,' I lines'
      if((qiron-nint(qiron)).ne.0.0)
     &   write(*,'(A23,A2,A9)')' Optimization based on ',
     &   xelem,' II lines'
      if(rj.ne.0.and.rj.ne.1)write(*,*)'Outliers rejection swicthed off'
      
      endif 
     
C--------------------------------------------------------------------- EXIT OPTIONS


      if(ewmin.ge.ewmax)then
        write(*,*)
	write(*,*)'WRONG EW LIMITS "ewmin"/"ewmax" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
    
         
      if(epot.ne.0.and.epot.ne.1)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "epot" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
      

      if(grc.ne.0.and.grc.ne.1.and.grc.ne.2.and.grc.ne.3)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "grc" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
 

      if(vtc.ne.0.and.vtc.ne.1)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "vtc" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
      
      
      if(qiron.lt.3.or.qiron.gt.99)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "qiron" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
      
      
      

C---------------------------------------------------------------------    
      
      !flag for the model
      ! 
      ! ym=0 alternative model
      ! ym=1 MARCS model
      ! ym=2 ATLAS9 model
      !
      ym=0
      do i=1,NMODM
        if(model.eq.atmos_marcs(i))ym=1
      enddo
      do i=1,NMODA
        if(model.eq.atmos_atlas(i))ym=2
      enddo
      
      
      if(ym.eq.0)then
       
       status = access(model,' ')
      
       if(status.ne.0)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "model" IN AUTOFL.PARAM'
	write(*,*)
	stop
       else
        grc=0
        epot=0
        vtc=0 
	metal=0
	guess=0
	refine=0
        step_true(1,1)=grc
        step_true(2,1)=epot
        step_true(3,1)=vtc
	call system('cp '//model//' atmmod.gala')  
	     	
       endif	
      endif
    
      
 
      if(uncer.ne.0.and.uncer.ne.1.and.uncer.ne.2)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "uncer" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif     
      
      
      if(refine.ne.0.and.refine.ne.1)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "refine" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
      
         
      if(paropt.ne.0.and.paropt.ne.1)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "paropt" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
       
         
      if(interz.ne.0.and.interz.ne.1)then
        write(*,*)
	write(*,*)'WRONG PARAMETER "interz" IN AUTOFL.PARAM'
	write(*,*)
	stop
      endif
      
      
      
 
C      
C---------------------------------------------------------------------   call the main procedure
C     
            

4     read(2,*,end=29)star,temp,gr,met,vmic,alfa,
     &uncer_t,uncer_g,uncer_vt 
      status = access(star(1:len_trim(star))//'.in' ,' ') 
      
      !------ initial atmospheric parameters 	 	 
      tinput=temp
      ginput=gr
      minput=met
      vinput=vmic
      

                
      if(status.ne.0) then 
       write(*,*)'Input file ',star(1:len_trim(star))//
     &           '.in does not exist!'
      else
       
       count0=0 
       count2=0
       count3=1
              
       grc=step_true(1,1)
       epot=step_true(2,1)
       vtc=step_true(3,1)
                        
       guess2=0
       x_err=0
       
       c_at=0
       c_avt=0
       c_ag=0
       
       modxxx=0
    
       
       if(verbose.eq.3)then 
       write(*,*)
       write(*,*)
     &'##############################################################'
       write(*,*)'                       STAR   ',star
       endif
       
       if(verbose.eq.1.or.verbose.eq.2)write(*,*)'STAR ',star
       
       
       inp=star(1:len_trim(star))//'.in' 
       outp=star(1:len_trim(star))//'.EW' 
       outabu=star(1:len_trim(star))//'.abu'       
       status = access(outp,' ')
       if(status.eq.0)call system('rm '//outp)         
       status = access(outabu,' ')
       if(status.eq.0)call system('rm '//outabu)  
       
       open(1,file=inp,status='unknown')
       num_lin=0
40     read(1,*,end=50) 
       num_lin=num_lin+1
       goto 40
50     continue      
       close(1) 
       
       !--- only for gravity from Stefan-Boltzmann or from Teff-logg relation
       if(grc.eq.2.or.grc.eq.3)call grav_ph(star,temp,gr)       
       
       call main(star,temp,gr,met,vmic,alfa,count0)
       
      endif
            
      outabu=star(1:len_trim(star))//'.abu'
      call system('cat '//outabu//' >> abundance.final')
      
      goto 4   
29    continue
     
      close(2)
      
      
      call system('rm fort.*')
      call system('rm *_gala.sh *jer atmmod.gala')
      if(ym.eq.1)call system('rm marcs_model*')
      if(ym.eq.2)call system('rm inmodel.at9')
      

C---------------------------------------------------------------------  GALAPLOT
      
      if(plot.ne.0)call system('galaplot')

C---------------------------------------------------------------------
      

      write(*,*)
      write(*,*)
      write(*,*)'          Now that the trees are gone, 
     &i can almost see the forest'
      write(*,*)'                                 
     &                      Gil Grissom'
      write(*,*)
      
      write(*,*)'------------------------------------
     &------------------------END OF GALA!' 
      
      
      
      end



      

