CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C									       C
C     This file is part of GALA					               C
C                                                                              C
C     GALA is free software: you can redistribute it and/or modify             C
C     it under the terms of the GNU General Public License as published by     C
C     the Free Software Foundation, either version 3 of the License, or        C
C     (at your option) any later version.                                      C
C                                                                              C
C     GALA is distributed in the hope that it will be useful,                  C
C     but WITHOUT ANY WARRANTY; without even the implied warranty of           C
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            C
C     GNU General Public License for more details.                             C
C    	                                                                       C
C     You should have received a copy of the GNU General Public License	       C
C     along with GALA. If not, see <http://www.gnu.org/licenses/>.             C
C									       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


      subroutine loopg(v,t,met,g,star,alfa,s,count0,nn)

C
C     grc = 0 fixed logg
C     grc = 1 logg optmized by imposing Fe~I = Fe~II
C     grc = 2 logg from photometry (Stefan-Boltzmann equation)
C     grc = 3 logg from a quadratic relation logg=A+B*Teff+C*Teff**2
C
C

      implicit none
      include 'Declare.f'           
               
      pday=4

C--------------------------------------------------------------------------------     
C     if the number of FeII lines is zero go to 'theend' fixing logg as the input value


C----- No log~g optimization is requested (grc=0), go to 'theend'     
C   
      if(grc.eq.0)then          
       if(verbose.ge.2)write(*,*)'GRAVITY FIXED!!!'
       
       
       if(x_err.eq.0)then
         call theend(star,t,g,met,v,alfa)
         return
       elseif(x_err.eq.1)then
	 param_p(1)=t
	 param_p(2)=g
	 param_p(3)=met
	 param_p(4)=v
	 param_p(5)=alfa
	 if(verbose.ge.2)
     &     write(*,*)'######################## 
     &                END ERROR COMPUTATION - 1'
	 return 	 
       elseif(x_err.eq.2)then
	 param_m(1)=t
	 param_m(2)=g
	 param_m(3)=met
	 param_m(4)=v
	 param_m(5)=alfa
	 if(verbose.ge.2)
     &     write(*,*)'######################## 
     &                END ERROR COMPUTATION - 2'
	 return
       elseif(x_err.eq.-1)then
         covar_p=v
         return 
       endif 
       
      endif
  
C--------------------------------------------------------------------------------           
C----- log~g computed by using the Stefan-Boltzmann equation, according with the Teff (grc=2)
C----- the factor logg-4*logTeff is read in the input list and a new value of logg is computed
C----- if the new logg is equal to the input value go to 'theend', otherwise go to 'main'


      if(grc.eq.2)then
       if(verbose.ge.2)write(*,*)'PHOTOMETRIC GRAVITY'
       call grav_ph(star,t,g_new)
 
       
       if(g.eq.g_new)then
       
          if(x_err.eq.0)then
            call theend(star,t,g,met,v,alfa)
	    return
          elseif(x_err.eq.1)then
	    param_p(1)=t
	    param_p(2)=g
	    param_p(3)=met
	    param_p(4)=v
	    param_p(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 1'
	   return 	 
          elseif(x_err.eq.2)then
	    param_m(1)=t
	    param_m(2)=g
	    param_m(3)=met
	    param_m(4)=v
	    param_m(5)=alfa
	    if(verbose.ge.2)
     &       write(*,*)'######################## 
     &                  END ERROR COMPUTATION - 2'
	    return
          endif 
	   
        else	   
          call main(star,t,g_new,met,v,alfa,count0)
       endif
       return
      endif
      
      
C--------------------------------------------------------------------------------           
C----- logg from a linear relation logg = A + (B * Teff) + (C * Teff**2), 
C------where A, B and C are read in the 'list_star'
      
      if(grc.eq.3)then
       if(verbose.ge.2)write(*,*)'GRAVITY FROM teff-logg RELATION'
       call grav_ph(star,t,g_new)
       
       if(g.eq.g_new)then
        
	 if(x_err.eq.0)then
           call theend(star,t,g,met,v,alfa)
	   return
	 elseif	(x_err.eq.1)then
	   param_p(1)=t
	   param_p(2)=g
	   param_p(3)=met
	   param_p(4)=v
	   param_p(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 1'
	   return 	 
         elseif(x_err.eq.2)then
	   param_m(1)=t
	   param_m(2)=g
	   param_m(3)=met
	   param_m(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 2'
	   return
         endif
	  
       else	   
         call main(star,t,g_new,met,v,alfa,count0)
       endif
       return
       
      endif
      
      
      
C------------------------------------------------------------
C---- logg optimization loop
       
      if(verbose.ge.2)write(*,*)'----------- CALL LOGG LOOP'  
      if(verbose.eq.3)then
       write(*,*)'Teff   logg    vt    [M/H]     d(Fe)   Jackknife 
     &    Merit'
      endif
      
      glam=0
      g_or=g
      stepgr=abs(stepgr)
      if(verbose.eq.3)write(*,410)t,g,v,met,s,jack_logg,merit
      
      
909   continue      
      g_new=g+stepgr
      g_new=int(g_new)+(nint(100.*(g_new-int(g_new))))/100.
      
      
C---- logg boundaries 0.0 --- 5.0
C---- MARCS grids with logg=-0.5 full of holes!
    
      
      if(g_new.gt.5.0.or.g_new.lt.0)then

         if(verbose.ge.2)
     &     write(*,*)'----------- END  LOGG LOOP (BOUNDARY GRID)'  
 
          if(x_err.eq.0)then
	   call theend(star,t,g,met,v,alfa)
	   return
	  elseif(x_err.eq.1)then
	   param_p(1)=t
	   param_p(2)=g
	   param_p(3)=met
	   param_p(4)=v
	   param_p(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 1'
	   return	   
	  elseif(x_err.eq.2)then
	   param_m(1)=t
	   param_m(2)=g
	   param_m(3)=met
	   param_m(4)=v
	   param_m(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 2'
	   return
	  endif  
	
      endif


C----------------------------------------------------------------------     
C---- compute the Fe~I abundance with log~g = log~g(main) + delta_log~g ]
      
      call xabu(star,t,g_new,met,v,alfa,2)   
      t=txxx
      g_new=grxxx

	
      if(grc.lt.2)difg=(mean-mean2)

C----- maximum iron difference allowed Fe~I-Fe~II=4.0 !!!!
      
      if(difg.gt.4)then
       grc=0
       call theend(star,t,g,met,v,alfa)
       return
      endif
      
      
      if(modxxx.eq.1)then
        
	  if(x_err.eq.0)then
	   call theend(star,t,g,met,v,alfa)
	   return
	  elseif(x_err.eq.1)then
	   param_p(1)=t
	   param_p(2)=g
	   param_p(3)=met
	   param_p(4)=v
	   param_p(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 1'
	   return	   
	  elseif(x_err.eq.2)then
	   param_m(1)=t
	   param_m(2)=g
	   param_m(3)=met
	   param_m(4)=v
	   param_m(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 2'
	   return
	  endif 
      
      
      endif
      
         
      if(verbose.eq.3)write(*,410)t,g_new,v,met,difg,jack_logg,merit
   
      
C---- optimization options based on the comparison between two iron differences
      
      if((s.lt.0.and.difg.gt.0).or.(s.gt.0.and.difg.lt.0))then
        difg1=difg
	if(g.gt.g_new)g_new2=g-stepgr
	if(g.lt.g_new)g_new2=g+stepgr
	
	if(abs(difg).gt.abs(s))then
          call xabu(star,t,g,met,v,alfa,2) 
	  t=txxx
	  g_new2=grxxx
	  difg1=(mean-mean2)   
	else
          call xabu(star,t,g_new,met,v,alfa,2) 
	  t=txxx
	  g_new2=grxxx
	  difg1=(mean-mean2)   
	endif 
	
	
       if(verbose.ge.2)write(*,*)'----------- END  LOGG LOOP' 
       
        if(g.eq.g_or)then
	  if(x_err.eq.0)then
	   call theend(star,t,g,met,v,alfa)
	   return
	  elseif(x_err.eq.1)then
	   param_p(1)=t
	   param_p(2)=g
	   param_p(3)=met
	   param_p(4)=v
	   param_p(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 1'
	   return	   
	  elseif(x_err.eq.2)then
	   param_m(1)=t
	   param_m(2)=g
	   param_m(3)=met
	   param_m(4)=v
	   param_m(5)=alfa
	   if(verbose.ge.2)
     &      write(*,*)'######################## 
     &                 END ERROR COMPUTATION - 2'
	   return
	  endif 
	else
          call repeat(t,g,met,v,alfa,ll,count0)
	  if(ll.eq.-1)call main(star,t,g,met,v,alfa,count0)
	  if(ll.eq.0)then
	   if(verbose.ge.2)write(*,*)'REPEATED MODEL!!!'
	   call theend(star,t,g,met,v,alfa)
	   return
	  endif
        endif
	   
	   
      elseif(s.gt.0.and.difg.gt.0)then
        nn=nn+1
        g=g_new
        s=difg
        goto 909
      elseif(s.lt.0.and.difg.lt.0)then
        nn=nn+1
        g=g_new
        s=difg
        stepgr=-abs(stepgr)
        goto 909
      endif
      

C-------------------------------------------------------------------  FORMATS

500   continue      

410   format(I5,3X,F4.2,3X,F4.2,3X,F5.2,3X,F7.3,4X,F7.3,3X,F8.2)  
            
      end subroutine
 
 
